=== modified file 'commit.py'
--- commit.py	2007-07-23 14:38:44 +0000
+++ commit.py	2007-09-27 22:16:26 +0000
@@ -39,6 +39,7 @@
 except ImportError:
     have_dbus = False
 
+
 class CommitDialog(gtk.Dialog):
     """ New implementation of the Commit dialog. """
     def __init__(self, wt, wtpath, notbranch, selected=None, parent=None):
@@ -85,6 +86,9 @@
                 self._is_pending = True
         
         # Create the widgets
+        # This is the main horizontal box, which is used to separate the commit
+        # info from the diff window.
+        self._hpane = gtk.HPaned()
         self._button_commit = gtk.Button(_("Comm_it"), use_underline=True)
         self._expander_files = gtk.Expander(_("File(s) to commit"))
         self._vpaned_main = gtk.VPaned()
@@ -103,7 +107,8 @@
 
         # Set callbacks
         self._button_commit.connect('clicked', self._on_commit_clicked)
-        self._treeview_files.connect('row_activated', self._on_treeview_files_row_activated)
+        self._treeview_files.connect('cursor-changed', self._on_treeview_files_cursor_changed)
+        self._treeview_files.connect('row-activated', self._on_treeview_files_row_activated)
         
         # Set properties
         self._scrolledwindow_files.set_policy(gtk.POLICY_AUTOMATIC,
@@ -142,7 +147,8 @@
 
         self._vpaned_main.add2(self._vbox_message)
         
-        self.vbox.pack_start(self._vpaned_main, True, True)
+        self._hpane.pack1(self._vpaned_main)
+        self.vbox.pack_start(self._hpane, expand=True, fill=True)
         if self._is_checkout: 
             self._check_local = gtk.CheckButton(_("_Only commit locally"),
                                                 use_underline=True)
@@ -165,6 +171,7 @@
         self._create_file_view()
         # Create the pending merges
         self._create_pending_merges()
+        self._create_diff_view()
         
         # Expand the corresponding expander
         if self._is_pending:
@@ -178,26 +185,20 @@
         # Default to Commit button
         self._button_commit.grab_default()
     
-    def _on_treeview_files_row_activated(self, treeview, path, view_column):
+    def _show_diff_view(self, treeview):
         # FIXME: the diff window freezes for some reason
         treeselection = treeview.get_selection()
         (model, iter) = treeselection.get_selected()
-        
+
         if iter is not None:
-            from diff import DiffWindow
-            
-            _selected = model.get_value(iter, 1)
-            
-            diff = DiffWindow()
-            diff.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
-            diff.set_modal(True)
-            parent_tree = self.wt.branch.repository.revision_tree(self.wt.branch.last_revision())
-            diff.set_diff(self.wt.branch.nick, self.wt, parent_tree)
-            try:
-                diff.set_file(_selected)
-            except errors.NoSuchFile:
-                pass
-            diff.show()
+            selected = model.get_value(iter, 3) # Get the real_path attribute
+            self._diff_display.show_diff([selected])
+
+    def _on_treeview_files_cursor_changed(self, treeview):
+        self._show_diff_view(treeview)
+        
+    def _on_treeview_files_row_activated(self, treeview, path, view_column):
+        self._show_diff_view(treeview)
     
     @show_bzr_error
     def _on_commit_clicked(self, button):
@@ -404,6 +405,16 @@
                                item['committer'],
                                item['summary'] ])
     
+
+    def _create_diff_view(self):
+        from diff import DiffDisplay
+
+        self._diff_display = DiffDisplay()
+        self._diff_display.set_trees(self.wt, self.wt.basis_tree())
+        self._diff_display.show_diff(None)
+        self._diff_display.show()
+        self._hpane.pack2(self._diff_display)
+
     def _get_specific_files(self):
         ret = []
         it = self._file_store.get_iter_first()

=== modified file 'diff.py'
--- diff.py	2007-09-17 20:20:43 +0000
+++ diff.py	2007-09-27 22:16:26 +0000
@@ -37,65 +37,19 @@
 from bzrlib.trace import warning
 
 
-class DiffWindow(gtk.Window):
-    """Diff window.
-
-    This object represents and manages a single window containing the
-    differences between two revisions on a branch.
-    """
+class DiffDisplay(gtk.ScrolledWindow):
+    """This is the soft and chewy filling for a DiffWindow."""
 
     def __init__(self):
-        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
-        self.set_border_width(0)
-        self.set_title("bzrk diff")
-
-        # Use two thirds of the screen by default
-        screen = self.get_screen()
-        monitor = screen.get_monitor_geometry(0)
-        width = int(monitor.width * 0.66)
-        height = int(monitor.height * 0.66)
-        self.set_default_size(width, height)
+        gtk.ScrolledWindow.__init__(self)
 
         self.construct()
+        self.rev_tree = None
+        self.parent_tree = None
 
     def construct(self):
-        """Construct the window contents."""
-        # The   window  consists  of   a  pane   containing:  the
-        # hierarchical list  of files on  the left, and  the diff
-        # for the currently selected file on the right.
-        pane = gtk.HPaned()
-        self.add(pane)
-        pane.show()
-
-        # The file hierarchy: a scrollable treeview
-        scrollwin = gtk.ScrolledWindow()
-        scrollwin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
-        scrollwin.set_shadow_type(gtk.SHADOW_IN)
-        pane.pack1(scrollwin)
-        scrollwin.show()
-
-        self.model = gtk.TreeStore(str, str)
-        self.treeview = gtk.TreeView(self.model)
-        self.treeview.set_headers_visible(False)
-        self.treeview.set_search_column(1)
-        self.treeview.connect("cursor-changed", self._treeview_cursor_cb)
-        scrollwin.add(self.treeview)
-        self.treeview.show()
-
-        cell = gtk.CellRendererText()
-        cell.set_property("width-chars", 20)
-        column = gtk.TreeViewColumn()
-        column.pack_start(cell, expand=True)
-        column.add_attribute(cell, "text", 0)
-        self.treeview.append_column(column)
-
-        # The diffs of the  selected file: a scrollable source or
-        # text view
-        scrollwin = gtk.ScrolledWindow()
-        scrollwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        scrollwin.set_shadow_type(gtk.SHADOW_IN)
-        pane.pack2(scrollwin)
-        scrollwin.show()
+        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.set_shadow_type(gtk.SHADOW_IN)
 
         if have_gtksourceview:
             self.buffer = gtksourceview.SourceBuffer()
@@ -114,78 +68,15 @@
 
         sourceview.set_editable(False)
         sourceview.modify_font(pango.FontDescription("Monospace"))
-        scrollwin.add(sourceview)
+        self.add(sourceview)
         sourceview.show()
 
-    def set_diff(self, description, rev_tree, parent_tree):
-        """Set the differences showed by this window.
-
-        Compares the two trees and populates the window with the
-        differences.
-        """
-        self.rev_tree = rev_tree
-        self.parent_tree = parent_tree
-
-        self.model.clear()
-        delta = self.rev_tree.changes_from(self.parent_tree)
-
-        self.model.append(None, [ "Complete Diff", "" ])
-
-        if len(delta.added):
-            titer = self.model.append(None, [ "Added", None ])
-            for path, id, kind in delta.added:
-                self.model.append(titer, [ path, path ])
-
-        if len(delta.removed):
-            titer = self.model.append(None, [ "Removed", None ])
-            for path, id, kind in delta.removed:
-                self.model.append(titer, [ path, path ])
-
-        if len(delta.renamed):
-            titer = self.model.append(None, [ "Renamed", None ])
-            for oldpath, newpath, id, kind, text_modified, meta_modified \
-                    in delta.renamed:
-                self.model.append(titer, [ oldpath, newpath ])
-
-        if len(delta.modified):
-            titer = self.model.append(None, [ "Modified", None ])
-            for path, id, kind, text_modified, meta_modified in delta.modified:
-                self.model.append(titer, [ path, path ])
-
-        self.treeview.expand_all()
-        self.set_title(description + " - bzrk diff")
-
-    def set_file(self, file_path):
-        tv_path = None
-        for data in self.model:
-            for child in data.iterchildren():
-                if child[0] == file_path or child[1] == file_path:
-                    tv_path = child.path
-                    break
-        if tv_path is None:
-            raise NoSuchFile(file_path)
-        self.treeview.set_cursor(tv_path)
-        self.treeview.scroll_to_cell(tv_path)
-
-    def _treeview_cursor_cb(self, *args):
-        """Callback for when the treeview cursor changes."""
-        (path, col) = self.treeview.get_cursor()
-        specific_files = [ self.model[path][1] ]
-        if specific_files == [ None ]:
-            return
-        elif specific_files == [ "" ]:
-            specific_files = None
-
-        s = StringIO()
-        show_diff_trees(self.parent_tree, self.rev_tree, s, specific_files)
-        self.buffer.set_text(s.getvalue().decode(sys.getdefaultencoding(), 'replace'))
-
     @staticmethod
     def apply_gedit_colors(lang):
         """Set style for lang to that specified in gedit configuration.
 
         This method needs the gconf module.
-        
+
         :param lang: a gtksourceview.SourceLanguage object.
         """
         GEDIT_SYNTAX_PATH = '/apps/gedit-2/preferences/syntax_highlighting'
@@ -261,7 +152,7 @@
                 except IOError, e:
                     warning('could not open file %s: %s' % (f, str(e)))
                 else:
-                    colors.update(DiffWindow.parse_colordiffrc(f))
+                    colors.update(DiffDisplay.parse_colordiffrc(f))
                     f.close()
 
         if not colors:
@@ -303,7 +194,7 @@
     @staticmethod
     def parse_colordiffrc(fileobj):
         """Parse fileobj as a colordiff configuration file.
-        
+
         :return: A dict with the key -> value pairs.
         """
         colors = {}
@@ -316,3 +207,132 @@
             colors[key.strip()] = val.strip()
         return colors
 
+    def set_trees(self, rev_tree, parent_tree):
+        self.rev_tree = rev_tree
+        self.parent_tree = parent_tree
+
+    def show_diff(self, specific_files):
+        s = StringIO()
+        show_diff_trees(self.parent_tree, self.rev_tree, s, specific_files)
+        self.buffer.set_text(s.getvalue().decode(sys.getdefaultencoding(), 'replace'))
+
+
+class DiffWindow(gtk.Window):
+    """Diff window.
+
+    This object represents and manages a single window containing the
+    differences between two revisions on a branch.
+    """
+
+    def __init__(self):
+        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
+        self.set_border_width(0)
+        self.set_title("bzrk diff")
+
+        # Use two thirds of the screen by default
+        screen = self.get_screen()
+        monitor = screen.get_monitor_geometry(0)
+        width = int(monitor.width * 0.66)
+        height = int(monitor.height * 0.66)
+        self.set_default_size(width, height)
+
+        self.construct()
+
+    def construct(self):
+        """Construct the window contents."""
+        # The   window  consists  of   a  pane   containing:  the
+        # hierarchical list  of files on  the left, and  the diff
+        # for the currently selected file on the right.
+        pane = gtk.HPaned()
+        self.add(pane)
+        pane.show()
+
+        # The file hierarchy: a scrollable treeview
+        scrollwin = gtk.ScrolledWindow()
+        scrollwin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        scrollwin.set_shadow_type(gtk.SHADOW_IN)
+        pane.pack1(scrollwin)
+        scrollwin.show()
+
+        self.model = gtk.TreeStore(str, str)
+        self.treeview = gtk.TreeView(self.model)
+        self.treeview.set_headers_visible(False)
+        self.treeview.set_search_column(1)
+        self.treeview.connect("cursor-changed", self._treeview_cursor_cb)
+        scrollwin.add(self.treeview)
+        self.treeview.show()
+
+        cell = gtk.CellRendererText()
+        cell.set_property("width-chars", 20)
+        column = gtk.TreeViewColumn()
+        column.pack_start(cell, expand=True)
+        column.add_attribute(cell, "text", 0)
+        self.treeview.append_column(column)
+
+        # The diffs of the  selected file: a scrollable source or
+        # text view
+        self.diff_win = DiffDisplay()
+        pane.pack2(self.diff_win)
+        self.diff_win.show()
+
+    def set_diff(self, description, rev_tree, parent_tree):
+        """Set the differences showed by this window.
+
+        Compares the two trees and populates the window with the
+        differences.
+        """
+        self.diff_win.set_trees(rev_tree, parent_tree)
+        self.rev_tree = rev_tree
+        self.parent_tree = parent_tree
+
+        self.model.clear()
+        delta = self.rev_tree.changes_from(self.parent_tree)
+
+        self.model.append(None, [ "Complete Diff", "" ])
+
+        if len(delta.added):
+            titer = self.model.append(None, [ "Added", None ])
+            for path, id, kind in delta.added:
+                self.model.append(titer, [ path, path ])
+
+        if len(delta.removed):
+            titer = self.model.append(None, [ "Removed", None ])
+            for path, id, kind in delta.removed:
+                self.model.append(titer, [ path, path ])
+
+        if len(delta.renamed):
+            titer = self.model.append(None, [ "Renamed", None ])
+            for oldpath, newpath, id, kind, text_modified, meta_modified \
+                    in delta.renamed:
+                self.model.append(titer, [ oldpath, newpath ])
+
+        if len(delta.modified):
+            titer = self.model.append(None, [ "Modified", None ])
+            for path, id, kind, text_modified, meta_modified in delta.modified:
+                self.model.append(titer, [ path, path ])
+
+        self.treeview.expand_all()
+        self.set_title(description + " - bzrk diff")
+
+    def set_file(self, file_path):
+        tv_path = None
+        for data in self.model:
+            for child in data.iterchildren():
+                if child[0] == file_path or child[1] == file_path:
+                    tv_path = child.path
+                    break
+        if tv_path is None:
+            raise NoSuchFile(file_path)
+        self.treeview.set_cursor(tv_path)
+        self.treeview.scroll_to_cell(tv_path)
+
+    def _treeview_cursor_cb(self, *args):
+        """Callback for when the treeview cursor changes."""
+        (path, col) = self.treeview.get_cursor()
+        specific_files = [ self.model[path][1] ]
+        if specific_files == [ None ]:
+            return
+        elif specific_files == [ "" ]:
+            specific_files = None
+
+        self.diff_win.show_diff(specific_files)

